@@ -747,9 +747,8 @@ public class AJPv13Response {
          * Write string content and terminating '0'
          */
         if (strLength > 0) {
-            final char[] chars = strValue.toCharArray();
             for (int i = 0; i < strLength; i++) {
-                sink.write((byte) chars[i]);
+                sink.write((byte) strValue.charAt(i));
             }
         }
         sink.write(0);
@@ -762,12 +761,10 @@ public class AJPv13Response {
          * Write string content and terminating '0'
          */
         if (strLength > 0) {
-            final char[] chars = strValue.toCharArray();
             final byte[] bytes = new byte[strLength];
-            System.arraycopy(chars, 0, bytes, 0, strLength);
-            /*for (int i = 0; i < strLength; i++) {
-                bytes[i] = (byte) chars[i];
-            }*/
+            for (int i = 0; i < strLength; i++) {
+                bytes[i] = (byte) strValue.charAt(i);
+            }
             c = writeByteArray(bytes, byteArray, c);
         }
         byteArray[c] = 0;
