@@ -95,6 +95,7 @@ import com.openexchange.ajp13.watcher.AJPv13TaskMonitor;
 import com.openexchange.configuration.ServerConfig;
 import com.openexchange.configuration.ServerConfig.Property;
 import com.openexchange.java.Charsets;
+import com.openexchange.log.ForceLog;
 import com.openexchange.log.Log;
 import com.openexchange.log.LogProperties;
 import com.openexchange.log.Props;
@@ -717,7 +718,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                 final Props properties = LogProperties.getLogProperties();
                 properties.put(LogProperties.Name.AJP_THREAD_NAME, thread.getName());
                 properties.put(LogProperties.Name.AJP_REMOTE_PORT, Integer.valueOf(socket.getPort()));
-                properties.put(LogProperties.Name.AJP_REMOTE_ADDRESS, socket.getInetAddress().getHostAddress());
+                properties.put(LogProperties.Name.AJP_REMOTE_ADDRESS, ForceLog.valueOf(socket.getInetAddress().getHostAddress()));
             }
             final boolean debug = LOG.isDebugEnabled();
             try {
@@ -841,14 +842,13 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                         final com.openexchange.java.StringAllocator sb = new com.openexchange.java.StringAllocator(512);
                         sb.append("400 - Bad Request: Error preparing forward-request: ").append(t.getClass().getName());
                         sb.append(" message=").append(t.getMessage());
-                        if (debug) {
-                            sb.append('\n');
-                            appendStackTrace(t.getStackTrace(), sb);
-                        }
-                        if (t instanceof RuntimeException) {
-                            LOG.warn(sb.toString(), t);
-                        } else {
+                        if (Log.appendTraceToMessage()) {
+                            final String lineSeparator = System.getProperty("line.separator");
+                            sb.append(lineSeparator);
+                            appendStackTrace(t.getStackTrace(), sb, lineSeparator);
                             LOG.warn(sb.toString());
+                        } else {
+                            LOG.warn(sb.toString(), t);
                         }
                     }
                     response.setStatus(400);
@@ -971,6 +971,9 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                     finish();
                 } catch (final IOException e) {
                     // Lost connection
+                    if (debug) {
+                        LOG.debug("Lost connection to web server.", e);
+                    }
                     error = true;
                 } catch (final Throwable t) {
                     ExceptionUtils.handleThrowable(t);
@@ -1079,6 +1082,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                 final Lock hardLock = mainLock.writeLock();
                 hardLock.lock();
                 try {
+                    boolean doReadMessage = true;
                     if (response.isCommitted()) {
                         /*
                          * Write empty SEND-BODY-CHUNK package
@@ -1099,18 +1103,40 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                         /*
                          * Receive probably empty body chunk
                          */
-                        final ByteChunk byteChunk = new ByteChunk(8192);
-                        final int read = inputBuffer.doRead(byteChunk, request);
-                        if (read > 0) {
-                            // Received a non-empty data chunk...
-                            // Dump that chunk to ServletInputStream
-                            int len = byteChunk.getLength();
-                            if (len > read) {
-                                len = read;
+                        if (doReadMessage) {
+                            // ... by separate AJP message
+                            final AjpMessage bodyMessage = new AjpMessage(packetSize);
+                            readMessage(bodyMessage);
+                            // No data received.
+                            if (bodyMessage.getLen() > 0) {
+                                final int blen = bodyMessage.peekInt();
+                                if (blen > 0) {
+                                    // Received a non-empty data chunk...
+                                    // Dump that chunk to ServletInputStream
+                                    final MessageBytes bodyBytes = MessageBytes.newInstance();
+                                    bodyMessage.getBytes(bodyBytes);
+                                    final ByteChunk bc = bodyBytes.getByteChunk();
+                                    final int len = bc.getLength();
+                                    final byte[] chunk = new byte[len];
+                                    System.arraycopy(bc.getBuffer(), bc.getStart(), chunk, 0, len);
+                                    request.appendToBuffer(chunk);
+                                }
+                            }
+                        } else {
+                            // ... by ByteChunk read attempt
+                            final ByteChunk byteChunk = new ByteChunk(8192);
+                            final int read = inputBuffer.doRead(byteChunk, request, false);
+                            if (read > 0) {
+                                // Received a non-empty data chunk...
+                                // Dump that chunk to ServletInputStream
+                                int len = byteChunk.getLength();
+                                if (len > read) {
+                                    len = read;
+                                }
+                                final byte[] chunk = new byte[len];
+                                System.arraycopy(byteChunk.getBuffer(), byteChunk.getStart(), chunk, 0, len);
+                                request.appendToBuffer(chunk);
                             }
-                            final byte[] chunk = new byte[len];
-                            System.arraycopy(byteChunk.getBuffer(), byteChunk.getStart(), chunk, 0, len);
-                            request.appendToBuffer(chunk);
                         }
                         if (DEBUG) {
                             LOG.debug("Performed keep-alive through an empty get-body-chunk package (and received requested chunk).");
@@ -1976,7 +2002,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
         {
             sink.reset();
             for (final Entry<String, List<String>> entry : response.getHeaderEntrySet()) {
-                for (String value : entry.getValue()) {
+                for (final String value : entry.getValue()) {
                     writeHeaderSafe(entry.getKey(), value, sink);
                     numHeaders++;
                 }
@@ -2221,6 +2247,14 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
          */
         @Override
         public int doRead(final ByteChunk chunk, final HttpServletRequestImpl req) throws IOException {
+            return doRead(chunk, req, true);
+        }
+
+        /**
+         * Read bytes into the specified chunk.
+         */
+        @Override
+        public int doRead(final ByteChunk chunk, final HttpServletRequestImpl req, final boolean refillIfEmpty) throws IOException {
             if (endOfStream) {
                 return -1;
             }
@@ -2232,7 +2266,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                     return 0;
                 }
             } else if (empty) {
-                if (!refillReadBuffer()) {
+                if (!refillIfEmpty || !refillReadBuffer()) {
                     return -1;
                 }
             }
@@ -2371,11 +2405,10 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
 
     } // End of class
 
-    private static void appendStackTrace(final StackTraceElement[] trace, final com.openexchange.java.StringAllocator sb) {
+    private static void appendStackTrace(final StackTraceElement[] trace, final com.openexchange.java.StringAllocator sb, final String lineSeparator) {
         if (null == trace) {
             return;
         }
-        final String lineSeparator = System.getProperty("line.separator");
         for (final StackTraceElement ste : trace) {
             final String className = ste.getClassName();
             if (null != className) {
